(function(global, $, name) {
    global[name] = function(options) {
        
        this.options = options;

        this.init = function() {
            this.container = this.options.el;
            this.slider = this.options.child;

            //Assume defaults for max, min, and value
            this.options.max = (this.options.max ? this.options.max : 1);
            this.options.min = (this.options.min ? this.options.min : 0);
            this.options.value = (this.options.value ? this.options.value : 0);
            this.options.disabled = (this.options.value ? this.options.value : false);
            this.options.callback = (this.options.callback ? this.options.callback : function() {});
            this.options.callbackArgs = (this.options.callbackArgs ? this.options.callbackArgs : []);

            //Determine the current width of the slider
            $(this.slider).width(this.calculateWidth());

            //Bind mouse handler to slider
            $(this.container).on("click", $.proxy(this.handleClick, this))
        }

        this.reset = function() {
            this.value(0);
        }

        //Calculate the current width based on min, max, width of
        //parent element, and current value
        this.calculateWidth = function() {
            total = this.options.max - this.options.min;

            if (total <= 0)
                return 0;

            percent = this.options.value / total;

            if (percent >= 1)
                return $(this.container).width();

            width = $(this.container).width() * percent;
            return width;
        }

        this.value = function(arg) {
            if (!arg)
                return this.options.value;

            this.options.value = arg;
            $(this.slider).width(this.calculateWidth());
        }

        this.min = function(arg) {
            if (!arg)
                return this.options.min;

            this.options.min = arg;
            $(this.slider).width(this.calculateWidth());
        }

        this.max = function(arg) {
            if (!arg)
                return this.options.max;

            this.options.max = arg;

            $(this.slider).width(this.calculateWidth());
        }

        this.handleClick = function(e) {
            if (this.disabled)
                return false;

            element = $(this.options.el);

            elementSize = {
                width: element.outerWidth(),
                height: element.outerHeight()
            }

            elementOffset = element.offset();
            position = {
                x: e.pageX,
                y: e.pageY
            }

            //Calculate the new width
            newWidth = position.x - elementOffset.left;
            $(this.slider).width(newWidth);

            //Calculate the new value
            value = (newWidth * (this.options.max - this.options.min))/element.width();

            if (value > this.options.max)
                value = this.options.max;
            else if (value < this.options.min)
                value = this.options.min;

            this.options.value = value;

            //Handle optional callback
            $.proxy(this.options.callback(this.options.callbackArgs), this);
        }
    }
})(window, jQuery, "EdnoiseSlider");
