(function(global, $, name) {
    function EdNoise() {

        this.el = null;
        this.playerTemplate = null;

        //Can only have one currently playing sound
        this.currentSound = null;
        this.currentSoundURL = null;

        //Need to generate custom ids
        this.soundCounter = 0;

        //Are we currently playing something?
        this.playing;
        //Are we currently muted?
        this.muted;
        this.volume;

        //Options for the player
        this.options = {
            markup: '\
                <div class = "player-container"> \
                    <div class = "play-pause"> \
                        <img src = "<%= imgs %>/play.png" /> \
                    </div> \
                    <div class = "time time-now">0:00</div> \
                    <div class = "scrubber-container"> \
                        <div class = "loaded"> \
                            <div class = "played"></div> \
                        </div> \
                    </div> \
                    <div class = "time time-total">0:00</div> \
                    <div class = "volume-icon"> \
                         <img src = "<%= imgs %>/audio-high.png" /> \
                    </div> \
                    <div class = "volume-slider-container"> \
                        <div class = "volume-slider"></div> \
                    </div> \
                </div>',

            imgs: "imgs",
        }
        
        //                        
        //Create a player given an element id
        this.create = function(el, imgs) {
            if (imgs)
                this.options.imgs = imgs;

            //Lazy initialize the player template
            if (!this.playerTemplate)
                this.playerTemplate = _.template(this.options.markup);


            $(el).append(this.playerTemplate(this.options));

            //Create the slider for the volume
            this.vslider = new EdnoiseSlider({
                el: ".volume-slider-container",
                child: ".volume-slider",
                min: 0,
                max: 100,
                value: 100,
                callback: function(args) {
                    //callbackArgs will be an array, the first element will
                    //be the current sound
                    if (args[0].currentSound)
                        args[0].currentSound.setVolume(this.value);

                    args[0].volume = this.value;
                },
                //Give this access to the global object to mess with sound
                callbackArgs: [global[name]]
            });

            this.vslider.init();

            //Create the slider for the scrubber
            this.sslider = new EdnoiseSlider({
                el: ".scrubber-container",
                child: ".played",
                min: 0,
                max: 0,
                value: 0,
                callback: function(args) {
                    soundManager.setPosition(args[0].currentSound.id, this.value);
                },
                callbackArgs: [global[name]]
            });

            this.sslider.init();

            //Save elements for later
            this.el = el;

            this.playPauseButton = el + " .play-pause img";
            this.volumeButton = el + " .volume-icon img";

            this.timeNow = el + " .time-now";
            this.timeTotal = el + " .time-total";

            this.loaded = el + " .loaded";
            this.played = el + " .played";

            //Attach event handlers
            $(this.playPauseButton).on("click",
                    $.proxy(this.playPause, global[name]));
            $(this.volumeButton).on("click",
                    $.proxy(this.muteUnmute, global[name]));

            this.muted = false;
            this.volume = 100;
        }
        
        this.playPause = function() {

            // If there is no current sound, can't do anything
            if (!this.currentSound)
                return
            
            if (this.playing) {
                this.currentSound.pause();
            }
            else {
                // Need to check if we can resume this song, or
                // if we should start playing from the beginning
                if (this.currentSound.position == null)
                    this.currentSound.play();
                else
                    this.currentSound.resume();
            }

            this.playing = !this.playing;

            //Set the contents of play/pause accordingly
            if (this.playing)
                $(this.playPauseButton).attr("src",
                        this.options.imgs + "/pause.png");
            else
                $(this.playPauseButton).attr("src",
                        this.options.imgs + "/play.png");
        }

        this.events = {
            play: function() {
                
            },

            pause: function() {
            },

            resume: function() {

            },

            stop: function() {

            },

            loaded: function() {
                $(this.timeTotal).text(
                    this.milisecondsToTime(
                        this.currentSound.duration));
                return;
            },

            loading: function() {
                //Store values for later use
                total = this.currentSound.duration;
                buffered = this.currentSound.buffered;

                //If everything wasn't loaded yet, proceding results
                //in errors
                if (!buffered[0] || typeof(buffered[0]) === undefined)
                    return;

                //WARNING: this is a chokepoint for multiple players being
                //initialized because buffered state is shared.  Needs to 
                //be handled later if multiple players are to become reality
                totalBuffered = buffered[0].end - buffered[0].start;

                //Calculate the width of the loaded bar given that it's parent is the scrubber-container
                $(this.loaded).width(
                    this.calculateWidth(total, totalBuffered, $(this.el + " .scrubber-container").width()));
            },
            
            playing: function() {
                //Store some values for later use
                total = this.currentSound.duration;
                played = this.currentSound.position;

                $(this.timeNow).text(
                    this.milisecondsToTime(played));

                this.sslider.max(total);
                this.sslider.value(played);
            },
        }

        this.reset = function() {
            this.sslider.reset();

            if (this.currentSound)
                this.currentSound.destruct();

            this.playing = false;
            $(this.timeNow).text(this.milisecondsToTime(0));
            $(this.timeTotal).text(this.milisecondsToTime(0));
        }

        this.load = function(url) {
            //Always do a reset on a load since we are getting something new
            this.reset();

            this.currentSound = soundManager.createSound({
                id: "ednoiseSound" + (this.soundCounter++),
                url: url,
                autoLoad: true,
                onplay: this.events.play,
                onpause: this.events.pause,
                onstop: this.events.stop,
                onfinish: $.proxy(this.events.trackEnded, global[name]),
                whileloading: $.proxy(this.events.loading, global[name]),
                whileplaying: $.proxy(this.events.playing, global[name]),
                onload: $.proxy(this.events.loaded, global[name])
            });
        }

        this.play = function(url) {
            //Need to destroy the current sound if there is one, create a new
            //sound, and play that one
            if (this.playing)
                this.reset();

            //Need to create a new sound
            if (!url && !this.currentSound)
                return;

            if (url)
                this.load(url);

            //set the current mute status and volume status
            this.setVolume(this.getVolume());

            if (this.muted)
                this.mute();
            else
                this.unmute();

            // Have to set this to false initially becuase currently
            // we are NOT playing anything until we hit play/pause.  Then
            // play/pause will flip it to true.
            this.playing = false;
            this.playPause();
        }

        this.pause = function() {
            if (global[name].playing)
                global[name].playPause();
        }

        this.resume = function() {
            if (!global[name].playing)
                global[name].playPause();
        }

        this.getVolume = function() {
            return this.volume;
        }

        this.setVolume = function(value) {
            // Some sanity checking
            if (value < 0 || value > 100)
                return;

            this.volume = value;
            global[name].vslider.value(this.volume);

            if (global[name].currentSound) {
                global[name].currentSound.setVolume(this.volume);
            }
        }

        // Need this helper function for calculating
        // duration of media 
        this.milisecondsToTime = function(ms) {
            seconds = ms / 1000;
            minutes = Math.floor(seconds / 60);
            seconds = Math.floor((seconds / 60 - minutes) * 60);

            return minutes + ":" + (seconds < 10 ? "0" + seconds : seconds);
        }

        //Calculate a width for a given element with a total, the amount currently
        //occupied, and the width of the parent element
        this.calculateWidth = function(total, done, parentElWidth) {

            //Just a basic proportion
            result = ((done * parentElWidth)/total) + "px";

            //If the result is greater than the parentElWidth (shouldn't happen)
            //just return the parent el width
            if (result >= parentElWidth)
                return parentElWidth;
            return result; 
        }

        this.mute = function() {
            //Reset the context
            if (global[name].currentSound)
                soundManager.mute(global[name].currentSound.id);

            //Change
            src = global[name].options.imgs + "/" + "audio-mute.png";
            $(global[name].volumeButton).attr("src", src);

            this.muted = true;
        }

        this.unmute = function() {
            if (global[name].currentSound)
                soundManager.unmute(global[name].currentSound.id);

            //Change
            src = global[name].options.imgs + "/" + "audio-high.png";
            $(global[name].volumeButton).attr("src", src);

            //Set state
            this.muted = false;
        }

        this.muteUnmute = function() {
            if (global[name].muted)
                this.unmute();
            else
                this.mute();
        }

    }

    //Clear out current instances
    var ednoise = null;

    //If this is IE, we need to not prefer flash because
    //it's HTML5 solution actually works somehow
    if (/MSIE (\d+\.\d+);/.test(navigator.userAgent)) {
        soundManager.setup({
            url: "static/ednoise/swf",
            preferFlash: false,
            flashVersion: 9
        });
    }
    else {
        soundManager.setup({
            url: "static/ednoise/swf",
            flashVersion: 9
        });
    }

    //Exports the ednoise variable
    soundManager.onready(function() {
      global[name] = new EdNoise("#ednoise");
    });

}(window, jQuery, "ednoise"));
